lists.openwall.net   lists  /  announce  owl-users  owl-dev  john-users  john-dev  passwdqc-users  yescrypt  popa3d-users  /  oss-security  kernel-hardening  musl  sabotage  tlsify  passwords  /  crypt-dev  xvendor  /  Bugtraq  Full-Disclosure  linux-kernel  linux-netdev  linux-ext4  linux-hardening  linux-cve-announce  PHC 
Open Source and information security mailing list archives
 
Hash Suite: Windows password security audit tool. GUI, reports in PDF.
[<prev] [next>] [<thread-prev] [thread-next>] [day] [month] [year] [list]
Date: Wed, 07 Feb 2024 16:10:55 +0100
From: Oleksandr Natalenko <oleksandr@...alenko.name>
To: rafael.j.wysocki@...el.com, Mario.Limonciello@....com,
 viresh.kumar@...aro.org, Ray.Huang@....com, gautham.shenoy@....com,
 Borislav.Petkov@....com, Perry Yuan <perry.yuan@....com>
Cc: Alexander.Deucher@....com, Xinmei.Huang@....com, Xiaojian.Du@....com,
 Li.Meng@....com, linux-pm@...r.kernel.org, linux-kernel@...r.kernel.org
Subject:
 Re: [PATCH v3 1/7] cpufreq: amd-pstate: initialize new core precision boost
 state

On středa 7. února 2024 15:45:20 CET Oleksandr Natalenko wrote:
> On středa 7. února 2024 10:21:52 CET Perry Yuan wrote:
> > From: Perry Yuan <Perry.Yuan@....com>
> > 
> > Add gloal global_params to represent current CPU Performance Boost(cpb)
> > state for cpu frequency scaling, both active and passive modes all can
> > support CPU cores frequency boosting control which is based on the BIOS
> > setting, while BIOS turn on the "Core Performance Boost", it will
> > allow OS control each core highest perf limitation from OS side.
> > 
> > If core performance boost is disabled while a core is in a boosted P-state,
> > the core transitions to the highest performance non-boosted P-state,
> > that is the same as the nominal frequency limit.
> > 
> > Reported-by: Artem S. Tashkinov" <aros@....com>
> > Closes: https://bugzilla.kernel.org/show_bug.cgi?id=217931
> > Signed-off-by: Perry Yuan <Perry.Yuan@....com>
> > ---
> >  drivers/cpufreq/amd-pstate.c | 49 ++++++++++++------------------------
> >  include/linux/amd-pstate.h   | 14 +++++++++++
> >  2 files changed, 30 insertions(+), 33 deletions(-)
> > 
> > diff --git a/drivers/cpufreq/amd-pstate.c b/drivers/cpufreq/amd-pstate.c
> > index 08e112444c27..3772f71f525f 100644
> > --- a/drivers/cpufreq/amd-pstate.c
> > +++ b/drivers/cpufreq/amd-pstate.c
> > @@ -67,6 +67,7 @@ static struct cpufreq_driver amd_pstate_epp_driver;
> >  static int cppc_state = AMD_PSTATE_UNDEFINED;
> >  static bool cppc_enabled;
> >  static bool amd_pstate_prefcore = true;
> > +struct global_params global;
> 
> Doesn't this require `EXPORT_SYMBOL_GPL(global)` then? Otherwise with v6.8 I get this:
> 
> ```
> ERROR: modpost: "global" [drivers/cpufreq/amd-pstate-ut.ko] undefined!
> ```
> 
> If so, I'd pretty mush rename this to be less generic, say, `amd_pstate_global_params_struct` for the struct name itself and `amd_pstate_global_params` for the actual variable name.

Something like this should do the trick (compile-tested only):

```
diff --git a/drivers/cpufreq/amd-pstate-ut.c b/drivers/cpufreq/amd-pstate-ut.c
index 91780ec42712..c5e2ca02f5ea 100644
--- a/drivers/cpufreq/amd-pstate-ut.c
+++ b/drivers/cpufreq/amd-pstate-ut.c
@@ -226,7 +226,7 @@ static void amd_pstate_ut_check_freq(u32 index)
 			goto skip_test;
 		}
 
-		if (global.cpb_supported) {
+		if (amd_pstate_global_params.cpb_supported) {
 			if ((policy->max == cpudata->max_freq) ||
 					(policy->max == cpudata->nominal_freq))
 				amd_pstate_ut_cases[index].result = AMD_PSTATE_UT_RESULT_PASS;
diff --git a/drivers/cpufreq/amd-pstate.c b/drivers/cpufreq/amd-pstate.c
index aebecd4e2e73..3e23d114c73f 100644
--- a/drivers/cpufreq/amd-pstate.c
+++ b/drivers/cpufreq/amd-pstate.c
@@ -68,7 +68,8 @@ static int cppc_state = AMD_PSTATE_UNDEFINED;
 static bool cppc_enabled;
 static bool amd_pstate_prefcore = true;
 static struct quirk_entry *quirks;
-struct global_params global;
+struct amd_pstate_global_params amd_pstate_global_params;
+EXPORT_SYMBOL_GPL(amd_pstate_global_params);
 
 /*
  * AMD Energy Preference Performance (EPP)
@@ -496,7 +497,7 @@ static void amd_pstate_update(struct amd_cpudata *cpudata, u32 min_perf,
 	value |= AMD_CPPC_DES_PERF(des_perf);
 
 	/* limit the max perf when core performance boost feature is disabled */
-	if (!global.cpb_boost)
+	if (!amd_pstate_global_params.cpb_boost)
 		max_perf = min_t(unsigned long, nominal_perf, max_perf);
 
 	value &= ~AMD_CPPC_MAX_PERF(~0L);
@@ -672,7 +673,7 @@ static int amd_get_max_freq(struct amd_cpudata *cpudata)
 	max_perf = READ_ONCE(cpudata->highest_perf);
 
 	/* when boost is off, the highest perf will be limited to nominal_perf */
-	if (!global.cpb_boost)
+	if (!amd_pstate_global_params.cpb_boost)
 		max_perf = nominal_perf;
 
 	boost_ratio = div_u64(max_perf << SCHED_CAPACITY_SHIFT,
@@ -737,8 +738,8 @@ static int amd_pstate_boost_init(struct amd_cpudata *cpudata)
 		return ret;
 	}
 
-	global.cpb_supported = !((boost_val >> 25) & 0x1);
-	global.cpb_boost = global.cpb_supported;
+	amd_pstate_global_params.cpb_supported = !((boost_val >> 25) & 0x1);
+	amd_pstate_global_params.cpb_boost = amd_pstate_global_params.cpb_supported;
 
 	return ret;
 }
@@ -1337,7 +1338,7 @@ static int amd_cpu_boost_update(struct amd_cpudata *cpudata, u32 on)
 static ssize_t cpb_boost_show(struct device *dev,
 			   struct device_attribute *attr, char *buf)
 {
-	return sysfs_emit(buf, "%u\n", global.cpb_boost);
+	return sysfs_emit(buf, "%u\n", amd_pstate_global_params.cpb_boost);
 }
 
 static ssize_t cpb_boost_store(struct device *dev, struct device_attribute *b,
@@ -1348,7 +1349,7 @@ static ssize_t cpb_boost_store(struct device *dev, struct device_attribute *b,
 	int cpu;
 
 	mutex_lock(&amd_pstate_driver_lock);
-	if (!global.cpb_supported) {
+	if (!amd_pstate_global_params.cpb_supported) {
 		pr_err("Boost mode is not supported by this processor or SBIOS\n");
 		return -EINVAL;
 	}
@@ -1357,7 +1358,7 @@ static ssize_t cpb_boost_store(struct device *dev, struct device_attribute *b,
 	if (ret)
 		return -EINVAL;
 
-	global.cpb_boost = !!new_state;
+	amd_pstate_global_params.cpb_boost = !!new_state;
 
 	for_each_possible_cpu(cpu) {
 
@@ -1371,7 +1372,7 @@ static ssize_t cpb_boost_store(struct device *dev, struct device_attribute *b,
 			goto err_exit;
 		}
 
-		amd_cpu_boost_update(cpudata, global.cpb_boost);
+		amd_cpu_boost_update(cpudata, amd_pstate_global_params.cpb_boost);
 		refresh_frequency_limits(policy);
 		cpufreq_cpu_put(policy);
 	}
diff --git a/include/linux/amd-pstate.h b/include/linux/amd-pstate.h
index b0db335f3883..f6e2c9825700 100644
--- a/include/linux/amd-pstate.h
+++ b/include/linux/amd-pstate.h
@@ -129,16 +129,16 @@ struct quirk_entry {
 };
 
 /**
  * struct global_params - Global parameters, mostly tunable via sysfs.
  * @cpb_boost:		Whether or not to use boost CPU P-states.
  * @cpb_supported:	Whether or not CPU boost P-states are available
  *			based on the MSR_K7_HWCR bit[25] state
  */
-struct global_params {
+struct amd_pstate_global_params {
 	bool cpb_boost;
 	bool cpb_supported;
 };
 
-extern struct global_params global;
+extern struct amd_pstate_global_params amd_pstate_global_params;
 
 #endif /* _LINUX_AMD_PSTATE_H */
@@ -129,16 +129,16 @@ struct quirk_entry {
 
 /**
- * struct global_params - Global parameters, mostly tunable via sysfs.
+ * struct amd_pstate_global_params - Global parameters, mostly tunable via sysfs.
  * @cpb_boost:		Whether or not to use boost CPU P-states.
  * @cpb_supported:	Whether or not CPU boost P-states are available
```

> >  
> >  /*
> >   * AMD Energy Preference Performance (EPP)
> > @@ -669,43 +670,21 @@ static int amd_get_lowest_nonlinear_freq(struct amd_cpudata *cpudata)
> >  	return lowest_nonlinear_freq * 1000;
> >  }
> >  
> > -static int amd_pstate_set_boost(struct cpufreq_policy *policy, int state)
> > +static int amd_pstate_boost_init(struct amd_cpudata *cpudata)
> >  {
> > -	struct amd_cpudata *cpudata = policy->driver_data;
> > +	u64 boost_val;
> >  	int ret;
> >  
> > -	if (!cpudata->boost_supported) {
> > -		pr_err("Boost mode is not supported by this processor or SBIOS\n");
> > -		return -EINVAL;
> > -	}
> > -
> > -	if (state)
> > -		policy->cpuinfo.max_freq = cpudata->max_freq;
> > -	else
> > -		policy->cpuinfo.max_freq = cpudata->nominal_freq;
> > -
> > -	policy->max = policy->cpuinfo.max_freq;
> > -
> > -	ret = freq_qos_update_request(&cpudata->req[1],
> > -				      policy->cpuinfo.max_freq);
> > -	if (ret < 0)
> > +	ret = rdmsrl_on_cpu(cpudata->cpu, MSR_K7_HWCR, &boost_val);
> > +	if (ret) {
> > +		pr_err_once("failed to read initial CPU boost state!\n");
> >  		return ret;
> > +	}
> >  
> > -	return 0;
> > -}
> > -
> > -static void amd_pstate_boost_init(struct amd_cpudata *cpudata)
> > -{
> > -	u32 highest_perf, nominal_perf;
> > -
> > -	highest_perf = READ_ONCE(cpudata->highest_perf);
> > -	nominal_perf = READ_ONCE(cpudata->nominal_perf);
> > -
> > -	if (highest_perf <= nominal_perf)
> > -		return;
> > +	global.cpb_supported = !((boost_val >> 25) & 0x1);
> > +	global.cpb_boost = global.cpb_supported;
> >  
> > -	cpudata->boost_supported = true;
> > -	current_pstate_driver->boost_enabled = true;
> > +	return ret;
> >  }
> >  
> >  static void amd_perf_ctl_reset(unsigned int cpu)
> > @@ -848,6 +827,9 @@ static int amd_pstate_cpu_init(struct cpufreq_policy *policy)
> >  	if (ret)
> >  		goto free_cpudata1;
> >  
> > +	/* initialize cpu cores boot state */
> > +	amd_pstate_boost_init(cpudata);
> > +
> >  	min_freq = amd_get_min_freq(cpudata);
> >  	max_freq = amd_get_max_freq(cpudata);
> >  	nominal_freq = amd_get_nominal_freq(cpudata);
> > @@ -899,7 +881,6 @@ static int amd_pstate_cpu_init(struct cpufreq_policy *policy)
> >  
> >  	policy->driver_data = cpudata;
> >  
> > -	amd_pstate_boost_init(cpudata);
> >  	if (!current_pstate_driver->adjust_perf)
> >  		current_pstate_driver->adjust_perf = amd_pstate_adjust_perf;
> >  
> > @@ -1310,6 +1291,9 @@ static int amd_pstate_epp_cpu_init(struct cpufreq_policy *policy)
> >  	if (ret)
> >  		goto free_cpudata1;
> >  
> > +	/* initialize cpu cores boot state */
> > +	amd_pstate_boost_init(cpudata);
> > +
> >  	min_freq = amd_get_min_freq(cpudata);
> >  	max_freq = amd_get_max_freq(cpudata);
> >  	nominal_freq = amd_get_nominal_freq(cpudata);
> > @@ -1360,7 +1344,6 @@ static int amd_pstate_epp_cpu_init(struct cpufreq_policy *policy)
> >  			return ret;
> >  		WRITE_ONCE(cpudata->cppc_cap1_cached, value);
> >  	}
> > -	amd_pstate_boost_init(cpudata);
> >  
> >  	return 0;
> >  
> > diff --git a/include/linux/amd-pstate.h b/include/linux/amd-pstate.h
> > index d21838835abd..465e9295a60c 100644
> > --- a/include/linux/amd-pstate.h
> > +++ b/include/linux/amd-pstate.h
> > @@ -124,4 +124,18 @@ static const char * const amd_pstate_mode_string[] = {
> >  	[AMD_PSTATE_GUIDED]      = "guided",
> >  	NULL,
> >  };
> > +
> > +/**
> > + * struct global_params - Global parameters, mostly tunable via sysfs.
> > + * @cpb_boost:		Whether or not to use boost CPU P-states.
> > + * @cpb_supported:	Whether or not CPU boost P-states are available
> > + *			based on the MSR_K7_HWCR bit[25] state
> > + */
> > +struct global_params {
> > +	bool cpb_boost;
> > +	bool cpb_supported;
> > +};
> > +
> > +extern struct global_params global;
> > +
> >  #endif /* _LINUX_AMD_PSTATE_H */
> > 
> 
> 
> 


-- 
Oleksandr Natalenko (post-factum)
Download attachment "signature.asc" of type "application/pgp-signature" (834 bytes)

Powered by blists - more mailing lists

Powered by Openwall GNU/*/Linux Powered by OpenVZ